import pytest

from slackminion.slack import SlackEvent, SlackChannel, SlackGroup, SlackIM, SlackUser
from slackminion.utils.test_helpers import *

test_im_id = 'D12345678'


# channel_id, channel_class
test_event_data = [
    (test_channel_id, SlackChannel),
    (test_group_id, SlackGroup),
    (test_im_id, SlackIM),
]


class TestSlackEvent(object):

    def test_init(self):
        event = SlackEvent()
        assert event._user is None
        assert event._channel is None

    def test_init_user(self):
        e = {'user': test_user_id}
        event = SlackEvent(**e)
        assert isinstance(event.user, SlackUser)
        assert event.channel is None

    def test_init_user_slackuser(self):
        e = {'user': SlackUser(test_user_id)}
        event = SlackEvent(**e)
        assert isinstance(event.user, SlackUser)
        assert event.channel is None

    @pytest.mark.parametrize('channel_id,channel_class', test_event_data)
    def test_init_channel(self, channel_id, channel_class):
        e = {'channel': channel_id}
        event = SlackEvent(**e)
        assert event.user is None
        assert isinstance(event.channel, channel_class)

    @pytest.mark.parametrize('channel_id,channel_class', test_event_data)
    def test_init_channel_slackchannel(self, channel_id, channel_class):
        e = {'channel': channel_class(channel_id)}
        event = SlackEvent(**e)
        assert event.user is None
        assert isinstance(event.channel, channel_class)



import pytest

from slackclient.channel import Channel
from slackclient.user import User
from slackminion.dispatcher import MessageDispatcher
from slackminion.exceptions import DuplicateCommandError
from slackminion.slack import SlackChannel
from slackminion.utils.test_helpers import *

test_data_mapping = []


@pytest.fixture(autouse=True)
def patch_slackclient_channels_find(monkeypatch):
    test_data_mapping.append(User(None, test_user_name, test_user_id, test_user_name, None, test_user_email))
    test_data_mapping.append(Channel(None, test_channel_name, test_channel_id, None))

    def find(self, id):
        res = filter(lambda x: x == id, test_data_mapping)
        if len(res) > 0:
            return res[0]
        return None
    monkeypatch.setattr('slackclient.util.SearchList.find', find)


class TestDispatcher(object):
    def setup(self):
        self.object = MessageDispatcher()
        self.p = DummyPlugin(None)

    def teardown(self):
        self.object = None

    def test_register_plugin(self):
        self.object.register_plugin(self.p)

    def test_register_duplicate_plugin(self):
        self.object.register_plugin(self.p)
        with pytest.raises(DuplicateCommandError) as e:
            self.object.register_plugin(self.p)
        assert 'abc' in str(e)

    def test_get_command(self):
        self.object.register_plugin(self.p)
        method = self.object._get_command('!abc', None).method
        assert method == self.p.abc
        assert method(None, None) == 'xyzzy'

    def test_get_invalid_command(self):
        self.object.register_plugin(self.p)
        with pytest.raises(KeyError):
            self.object._get_command('!def', None)

    def test_parse_message(self):
        e = SlackEvent(text='Hello world')
        assert self.object._parse_message(e) == ['Hello', 'world']

    def test_ignore_channel(self):
        assert self.object.ignore('testchannel') is True
        assert 'testchannel' in self.object.ignored_channels

    def test_unignore_channel(self):
        assert self.object.ignore('testchannel') is True
        assert self.object.unignore('testchannel') is True
        assert 'testchannel' not in self.object.ignored_channels

    def test_unignore_nonignored_channel(self):
        self.object.unignore('testchannel') is False
        assert 'testchannel' not in self.object.ignored_channels

    def test_ignore_duplicate_channel(self):
        assert self.object.ignore('testchannel') is True
        assert 'testchannel' in self.object.ignored_channels
        assert self.object.ignore('testchannel') is False
        assert len(self.object.ignored_channels) == 1

    def test_ignore_slackchannel(self):
        c = SlackChannel('CTEST')
        c.name = 'testchannel'
        assert self.object.ignore(c) is True
        assert 'testchannel' in self.object.ignored_channels
        assert self.object._is_channel_ignored(self.p.abc, c) is True

    def test_unignore_slackchannel(self):
        c = SlackChannel('CTEST')
        c.name = 'testchannel'
        assert self.object.ignore(c) is True
        assert self.object.unignore(c) is True
        assert 'testchannel' not in self.object.ignored_channels
        assert self.object._is_channel_ignored(self.p.abc, c) is False

    def test_push(self):
        self.object.register_plugin(self.p)
        e = SlackEvent(DummySlackConnection(), **{'text': '!abc', 'user': test_user_id, 'channel': test_channel_id})
        assert self.object.push(e) == ('!abc', 'xyzzy')

    def test_push_alias(self):
        self.object.register_plugin(self.p)
        e = SlackEvent(DummySlackConnection(), **{'text': '!xyz', 'user': test_user_id, 'channel': test_channel_id})
        assert self.object.push(e) == ('!xyz', 'xyzzy')

    def test_push_not_command(self):
        e = SlackEvent(DummySlackConnection(), **{'text': 'Not a command'})
        assert self.object.push(e) == (None, None)

    def test_push_message_replied_event(self):
        e = SlackEvent(DummySlackConnection(), **{'subtype': 'message_replied'})
        assert self.object.push(e) == (None, None)

    def test_push_no_user(self):
        self.object.register_plugin(self.p)
        e = SlackEvent(DummySlackConnection(), **{'text': '!abc'})
        assert self.object.push(e) == (None, None)

    def test_push_ignored_channel(self):
        c = SlackChannel('CTEST')
        c.name = 'testchannel'
        self.object.ignore(c)
        self.object.register_plugin(self.p)
        e = SlackEvent(DummySlackConnection(), **{'text': '!abc', 'user': test_user_id, 'channel': test_channel_id})
        assert self.object.push(e) == ('_ignored_', '')



import pytest

from slackclient import SlackClient
from slackclient.user import User

from slackminion.slack import SlackUser
from slackminion.utils.test_helpers import *

str_format = '<@{id}|{name}>'

test_user_mapping = []


@pytest.fixture(autouse=True)
def patch_slackclient_channels_find(monkeypatch):
    test_user_mapping.append(User(None, test_user_name, test_user_id, test_user_name, None, test_user_email))

    def find(self, id):
        users = filter(lambda x: x == id, test_user_mapping)
        if len(users) > 0:
            return users[0]
        return None
    monkeypatch.setattr('slackclient.util.SearchList.find', find)


class TestSlackUser(object):

    def setup(self):
        self.object = SlackUser(test_user_id, sc=DummySlackConnection())

    def teardown(self):
        self.object = None

    def test_init(self):
        assert self.object.id == test_user_id

    def test_userid(self):
        assert self.object.userid == test_user_id

    def test_username(self):
        assert self.object.username == test_user_name

    def test_str(self):
        assert str(self.object) == str_format.format(id=test_user_id, name=test_user_name)

    def test_repr(self):
        assert repr(self.object) == test_user_id

    def test_get_user(self):
        user = SlackUser.get_user(DummySlackConnection(), test_user_name)
        assert isinstance(user, SlackUser)


def test_get_user_none():
    user = SlackUser.get_user(DummySlackConnection(), 'doesnotexist')
    assert user is None



import pytest
import yaml

from slackminion.bot import Bot
from slackminion.exceptions import NotSetupError
from slackminion.plugins.core import version


class TestBot(object):
    def setup(self):
        with open('config.yaml.example', 'r') as f:
            self.object = Bot(config=yaml.load(f), test_mode=True)

    def teardown(self):
        self.object = None

    def test_init(self):
        assert self.object.version == version
        assert self.object.commit == 'HEAD'

    def test_start(self):
        self.object.start()
        assert self.object.is_setup is True

    def test_stop(self):
        self.object.stop()

    def test_run_without_start(self):
        with pytest.raises(NotSetupError) as e:
            self.object.run()
        assert 'Bot not setup' in str(e)

    def test_handler_slack_reconnect_event(self):
        self.object.reconnect_needed = False
        self.object._event_team_migration_started(None)
        assert self.object.reconnect_needed is True
